@@ -1,7 +1,7 @@
 // qtractorAudioEngine.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2017, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2018, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -116,7 +116,42 @@ static inline void sse_buffer_add (
 	}
 }
 
-#endif
+#endif // __SSE__
+
+
+#if defined(__ARM_NEON__)
+
+#include "arm_neon.h"
+
+// NEON enabled mix-down processor version.
+static inline void neon_buffer_add (
+	float **ppBuffer, float **ppFrames, unsigned int iFrames,
+	unsigned short iBuffers, unsigned short iChannels, unsigned int iOffset )
+{
+	unsigned short j = 0;
+
+	for (unsigned short i = 0; i < iChannels; ++i) {
+		float *pBuffer = ppBuffer[j] + iOffset;
+		float *pFrames = ppFrames[i] + iOffset;
+		unsigned int nframes = iFrames;
+		for (; (long(pBuffer) & 15) && (nframes > 0); --nframes)
+			*pBuffer++ += *pFrames++;
+		for (; nframes >= 4; nframes -= 4) {
+			vst1q_f32(pBuffer,
+				vaddq_f32(
+					vld1q_f32(pBuffer),
+					vld1q_f32(pFrames)));
+			pFrames += 4;
+			pBuffer += 4;
+		}
+		for (; nframes > 0; --nframes)
+			*pBuffer++ += *pFrames++;
+		if (++j >= iBuffers)
+			j = 0;
+	}
+}
+
+#endif // __ARM_NEON__
 
 
 // Standard mix-down processor version.
@@ -162,7 +197,11 @@ public:
 			m_pfnBufferAdd = sse_buffer_add;
 		else
 	#endif
-		m_pfnBufferAdd = std_buffer_add;
+	#if defined(__ARM_NEON__)
+		m_pfnBufferAdd = neon_buffer_add;
+		if (false)
+	#endif
+			m_pfnBufferAdd = std_buffer_add;
 	}
 
 	// Destructor.
@@ -284,10 +323,11 @@ static int qtractorAudioEngine_graph_order ( void *pvArg )
 
 
 //----------------------------------------------------------------------
-// qtractorAudioEngine_graph_port -- JACK port registration callback.
+// qtractorAudioEngine_client_registration -- JACK client reg. callback.
 //
 
-static void qtractorAudioEngine_graph_port ( jack_port_id_t, int, void *pvArg )
+static void qtractorAudioEngine_client_registration (
+	const char *, int, void *pvArg )
 {
 	qtractorAudioEngine *pAudioEngine
 		= static_cast<qtractorAudioEngine *> (pvArg);
@@ -296,6 +336,52 @@ static void qtractorAudioEngine_graph_port ( jack_port_id_t, int, void *pvArg )
 }
 
 
+//----------------------------------------------------------------------
+// qtractorAudioEngine_port_registration -- JACK port reg. callback.
+//
+
+static void qtractorAudioEngine_port_registration (
+	jack_port_id_t, int, void *pvArg )
+{
+	qtractorAudioEngine *pAudioEngine
+		= static_cast<qtractorAudioEngine *> (pvArg);
+
+	pAudioEngine->notifyPortEvent();
+}
+
+
+//----------------------------------------------------------------------
+// qtractorAudioEngine_port_connect -- JACK port conn. callback.
+//
+
+static void qtractorAudioEngine_port_connect (
+	jack_port_id_t, jack_port_id_t, int, void *pvArg )
+{
+	qtractorAudioEngine *pAudioEngine
+		= static_cast<qtractorAudioEngine *> (pvArg);
+
+	pAudioEngine->notifyPortEvent();
+}
+
+
+#ifdef CONFIG_JACK_PORT_RENAME
+
+//----------------------------------------------------------------------
+// qtractorAudioEngine_port_rename -- JACK port rename callback.
+//
+
+static void qtractorAudioEngine_port_rename (
+	jack_port_id_t, const char *, const char *, void *pvArg )
+{
+	qtractorAudioEngine *pAudioEngine
+		= static_cast<qtractorAudioEngine *> (pvArg);
+
+	pAudioEngine->notifyPortEvent();
+}
+
+#endif
+
+
 //----------------------------------------------------------------------
 // qtractorAudioEngine_buffer_size -- JACK buffer-size change callback.
 //
@@ -630,8 +716,16 @@ bool qtractorAudioEngine::activate (void)
 		qtractorAudioEngine_shutdown, this);
 	jack_set_graph_order_callback(m_pJackClient,
 		qtractorAudioEngine_graph_order, this);
+	jack_set_client_registration_callback(m_pJackClient,
+		qtractorAudioEngine_client_registration, this);
 	jack_set_port_registration_callback(m_pJackClient,
-		qtractorAudioEngine_graph_port, this);
+		qtractorAudioEngine_port_registration, this);
+	jack_set_port_connect_callback(m_pJackClient,
+		qtractorAudioEngine_port_connect, this);
+#ifdef CONFIG_JACK_PORT_RENAME
+	jack_set_port_rename_callback(m_pJackClient,
+		qtractorAudioEngine_port_rename, this);
+#endif
 	jack_set_buffer_size_callback(m_pJackClient,
 		qtractorAudioEngine_buffer_size, this);
 
@@ -2156,7 +2250,11 @@ qtractorAudioBus::qtractorAudioBus (
 		m_pfnBufferAdd = sse_buffer_add;
 	else
 #endif
-	m_pfnBufferAdd = std_buffer_add;
+#if defined(__ARM_NEON__)
+	m_pfnBufferAdd = neon_buffer_add;
+	if (false)
+#endif
+		m_pfnBufferAdd = std_buffer_add;
 }
 
 
