@@ -1,7 +1,7 @@
 // qtractorMidiControlObserver.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2016, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2018, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -60,7 +60,7 @@ qtractorMidiControlObserver::qtractorMidiControlObserver (
 	qtractorSubject *pSubject ) : qtractorObserver(pSubject),
 		m_ctype(qtractorMidiEvent::CONTROLLER), m_iChannel(0), m_iParam(0),
 		m_bLogarithmic(false), m_bFeedback(false),
-		m_bInvert(false), m_bHook(false), m_bLatch(true),
+		m_bInvert(false), m_bHook(false), m_bLatch(true), m_bTriggered(false),
 		m_fMidiValue(0.0f), m_bMidiSync(false), m_pCurveList(NULL)
 {
 }
@@ -79,6 +79,7 @@ qtractorMidiControlObserver::~qtractorMidiControlObserver (void)
 unsigned short qtractorMidiControlObserver::midiScale (void) const
 {
 	if (m_ctype == qtractorMidiEvent::PITCHBEND ||
+
 		m_ctype == qtractorMidiEvent::CONTROL14 ||
 		m_ctype == qtractorMidiEvent::REGPARAM  ||
 		m_ctype == qtractorMidiEvent::NONREGPARAM)
@@ -100,14 +101,22 @@ void qtractorMidiControlObserver::setMidiValue ( unsigned short iValue )
 		= float(m_bInvert ? iScale - iValue : iValue) / float(iScale);
 	float fValue = valueFromScale(fScale, m_bLogarithmic);
 
-	if (!m_bLatch && qtractorObserver::isToggled()) {
+	const bool bToggled = qtractorObserver::isToggled();
+
+	if (bToggled || m_bTriggered) {
 		const float vmax = qtractorObserver::maxValue();
 		const float vmin = qtractorObserver::minValue();
 		const float vmid = 0.5f * (vmax + vmin);
+		if (bToggled && m_bLatch) // latched / non-momentary...
+			fValue = (fValue > vmid ? vmax : vmin);
+		else // momentary / non-latched...
 		if (fValue > vmid)
 			fValue = (m_fMidiValue > vmid ? vmin : vmax);
 		else
+		if (fValue > vmin)
 			fValue = (m_fMidiValue < vmid ? vmax : vmin);
+		else
+			fValue = m_fMidiValue;
 	}
 
 	bool bSync = (m_bHook || !qtractorObserver::isDecimal());
