@@ -1,7 +1,7 @@
 // qtractorMidiManager.h
 //
 /****************************************************************************
-   Copyright (C) 2005-2017, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2018, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -327,7 +327,8 @@ protected:
 	// Process/decode into other/plugin event buffers...
 	void processEventBuffers();
 
-	// Swap event buffers (in for out and vice-versa)
+	// Reset/swap event buffers (in for out and vice-versa)
+	void resetEventBuffers();
 	void swapEventBuffers();
 
 private:
