@@ -1,7 +1,7 @@
 // qtractorPluginForm.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2017, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2018, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -546,27 +546,33 @@ void qtractorPluginForm::openPresetSlot (void)
 
 	// Prompt if file does not currently exist...
 	const QString sExt("qtx");
-	QStringList filters(sExt);
+	QStringList exts(sExt);
 	if (bVstPlugin) {
-		filters.append("fxp");
-		filters.append("fxb");
+		exts.append("fxp");
+		exts.append("fxb");
 	}
 
 	const QString& sTitle
 		= tr("Open Preset") + " - " QTRACTOR_TITLE;
-	const QString& sFilter
-		= tr("Preset files (*.%1)").arg(filters.join(" *."));
 
+	QStringList filters;
+	filters.append(tr("Preset files (*.%1)").arg(exts.join(" *.")));
+	filters.append(tr("All files (*.*)"));
+	const QString& sFilter = filters.join(";;");
+
+	QWidget *pParentWidget = NULL;
 	QFileDialog::Options options = 0;
-	if (pOptions->bDontUseNativeDialogs)
+	if (pOptions->bDontUseNativeDialogs) {
 		options |= QFileDialog::DontUseNativeDialog;
+		pParentWidget = QWidget::window();
+	}
 #if 1//QT_VERSION < 0x040400
 	// Ask for the filename to save...
-	sFilename = QFileDialog::getOpenFileName(this,
+	sFilename = QFileDialog::getOpenFileName(pParentWidget,
 		sTitle, pOptions->sPresetDir, sFilter, NULL, options);
 #else
 	// Construct save-file dialog...
-	QFileDialog fileDialog(this,
+	QFileDialog fileDialog(pParentWidget,
 		sTitle, pOptions->sPresetDir, sFilter);
 	// Set proper open-file modes...
 	fileDialog.setAcceptMode(QFileDialog::AcceptOpen);
@@ -636,8 +642,8 @@ void qtractorPluginForm::savePresetSlot (void)
 		// Sure, we'll have something complex enough
 		// to make it save into an external file...
 		const QString sExt("qtx");
-		QStringList filters(sExt);
-		if (bVstPlugin) { filters.append("fxp"); filters.append("fxb"); }
+		QStringList exts(sExt);
+		if (bVstPlugin) { exts.append("fxp"); exts.append("fxb"); }
 		QFileInfo fi(settings.value(sPreset).toString());
 		if (!fi.exists() || !fi.isFile())
 			fi.setFile(QDir(pOptions->sPresetDir),
@@ -647,18 +653,23 @@ void qtractorPluginForm::savePresetSlot (void)
 		if (!fi.exists()) {
 			const QString& sTitle
 				= tr("Save Preset") + " - " QTRACTOR_TITLE;
-			const QString& sFilter
-				= tr("Preset files (*.%1)").arg(filters.join(" *."));
+			QStringList filters;
+			filters.append(tr("Preset files (*.%1)").arg(exts.join(" *.")));
+			filters.append(tr("All files (*.*)"));
+			const QString& sFilter = filters.join(";;");
+			QWidget *pParentWidget = NULL;
 			QFileDialog::Options options = 0;
-			if (pOptions->bDontUseNativeDialogs)
+			if (pOptions->bDontUseNativeDialogs) {
 				options |= QFileDialog::DontUseNativeDialog;
+				pParentWidget = QWidget::window();
+			}
 		#if 1//QT_VERSION < 0x040400
 			// Ask for the filename to save...
-			sFilename = QFileDialog::getSaveFileName(this,
+			sFilename = QFileDialog::getSaveFileName(pParentWidget,
 				sTitle, sFilename, sFilter, NULL, options);
 		#else
 			// Construct save-file dialog...
-			QFileDialog fileDialog(this,
+			QFileDialog fileDialog(pParentWidget,
 				sTitle, sFilename, sFilter);
 			// Set proper open-file modes...
 			fileDialog.setAcceptMode(QFileDialog::AcceptSave);
@@ -1511,15 +1522,18 @@ void qtractorPluginPropertyWidget::buttonClicked (void)
 	const QString& sTitle
 		= tr("Open File") + " - " QTRACTOR_TITLE;
 
+	QWidget *pParentWidget = NULL;
 	QFileDialog::Options options = 0;
-	if (pOptions->bDontUseNativeDialogs)
+	if (pOptions->bDontUseNativeDialogs) {
 		options |= QFileDialog::DontUseNativeDialog;
+		pParentWidget = QWidget::window();
+	}
 #if 1//QT_VERSION < 0x040400
-	sFilename = QFileDialog::getOpenFileName(this,
+	sFilename = QFileDialog::getOpenFileName(pParentWidget,
 		sTitle, sFilename, QString(), NULL, options);
 #else
 	// Construct open-file dialog...
-	QFileDialog fileDialog(this, sTitle, sFilename);
+	QFileDialog fileDialog(pParentWidget, sTitle, sFilename);
 	// Set proper open-file modes...
 	fileDialog.setAcceptMode(QFileDialog::AcceptOpen);
 	fileDialog.setFileMode(QFileDialog::ExistingFile);
@@ -1585,7 +1599,7 @@ bool qtractorPluginPropertyWidget::eventFilter (
 	QObject *pObject, QEvent *pEvent )
 {
 	if (qobject_cast<QTextEdit *> (pObject) == m_pTextEdit) {
-		if(pEvent->type() == QEvent::KeyPress) {
+		if (pEvent->type() == QEvent::KeyPress) {
 			QKeyEvent *pKeyEvent = static_cast<QKeyEvent*> (pEvent);
 			if (pKeyEvent->key() == Qt::Key_Return
 				&& (pKeyEvent->modifiers() &
@@ -1595,7 +1609,7 @@ bool qtractorPluginPropertyWidget::eventFilter (
 			}
 		}
 		else
-		if(pEvent->type() == QEvent::FocusOut)
+		if (pEvent->type() == QEvent::FocusOut)
 			propertyChanged();
 	}
 
