@@ -1,7 +1,7 @@
 // qtractorPluginListView.h
 //
 /****************************************************************************
-   Copyright (C) 2005-2017, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2018, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -217,6 +217,9 @@ protected:
 	// Direct access parameter handle.
 	void dragDirectAccess(const QPoint& pos);
 
+	// Direct access parameter reset (to default value).
+	void resetDirectAccess(const QPoint& pos);
+
 	// Initial size-policy hints.
 	QSize sizeHint() const;
 
