@@ -1,7 +1,7 @@
 // qtractorSession.h
 //
 /****************************************************************************
-   Copyright (C) 2005-2017, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2018, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -301,7 +301,7 @@ public:
 
 	// Create a brand new filename (absolute file path).
 	QString createFilePath(
-		const QString& sBaseName, const QString& sExt);
+		const QString& sBaseName, const QString& sExt, bool bAcquire = false);
 
 	// Consolidated session record state.
 	void setRecording(bool bRecording);
@@ -321,6 +321,11 @@ public:
 	void trackMute(qtractorTrack *pTrack, bool bMute);
 	void trackSolo(qtractorTrack *pTrack, bool bSolo);
 
+	// Special auto-plugin-deactivation
+	void autoDeactivatePlugins(bool bForce = false);
+	void setAutoDeactivate(bool bOn);
+	bool isAutoDeactivate() const;
+
 	// Audio peak factory accessor.
 	qtractorAudioPeakFactory *audioPeakFactory() const;
 
@@ -407,6 +412,12 @@ public:
 
 private:
 
+	// check if plugin can be auto deactivated
+	// for now private - maybe helpful for others?
+	bool canTrackBeAutoDeactivated(qtractorTrack *pTrack) const;
+	// Restore activation state
+	void undoAutoDeactivatePlugins();
+
 	Properties     m_props;             // Session properties.
 
 	unsigned long  m_iSessionStart;     // Session start in frames.
@@ -474,6 +485,9 @@ private:
 	// Auto time-stretching global flag (when tempo changes)
 	bool m_bAutoTimeStretch;
 
+	// Auto disable plugins flag
+	bool m_bAutoDeactivate;
+
 	// MIDI plugin manager list.
 	qtractorList<qtractorMidiManager> m_midiManagers;
 
