@@ -1,7 +1,7 @@
 // qtractorTrackCommand.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2017, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2018, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -82,6 +82,10 @@ bool qtractorTrackCommand::addTrack (void)
 	if (pTracks == NULL)
 		return false;
 
+	qtractorTrackList *pTrackList = pTracks->trackList();
+	if (pTrackList == NULL)
+		return false;
+
 	// Guess which item we're adding after...
 #if 0
 	if (m_pAfterTrack == NULL)
@@ -93,9 +97,10 @@ bool qtractorTrackCommand::addTrack (void)
 	int iTrack = pSession->tracks().find(m_pAfterTrack) + 1;
 	// Link the track into session...
 	pSession->insertTrack(m_pTrack, m_pAfterTrack);
+
 	// And the new track list view item too...
-	qtractorTrackList *pTrackList = pTracks->trackList();
 	iTrack = pTrackList->insertTrack(iTrack, m_pTrack);
+
 	// Special MIDI track cases...
 	if (m_pTrack->trackType() == qtractorTrack::Midi)
 	    pTracks->updateMidiTrack(m_pTrack);
@@ -146,6 +151,10 @@ bool qtractorTrackCommand::removeTrack (void)
 	if (pTracks == NULL)
 		return false;
 
+	qtractorTrackList *pTrackList = pTracks->trackList();
+	if (pTrackList == NULL)
+		return false;
+
 	// Save which item we're adding after...
 	if (m_pAfterTrack == NULL)
 		m_pAfterTrack = m_pTrack->prev();
@@ -167,16 +176,16 @@ bool qtractorTrackCommand::removeTrack (void)
 	pSession->unlinkTrack(m_pTrack);
 
 	// Third, remove track from list view...
-	qtractorTrackList *pTrackList = pTracks->trackList();
 	iTrack = pTrackList->removeTrack(iTrack);
-	if (iTrack >= 0)
-		pTrackList->setCurrentTrackRow(iTrack);
 
 	// Mixer turn...
 	qtractorMixer *pMixer = pMainForm->mixer();
 	if (pMixer)
 		pMixer->updateTracks();
 
+	// Let the change get visible.
+	pTrackList->setCurrentTrackRow(iTrack);
+
 	// ATTN: MIDI controller map feedback.
 	qtractorMidiControl *pMidiControl = qtractorMidiControl::getInstance();
 	if (pMidiControl)
@@ -308,6 +317,10 @@ bool qtractorCopyTrackCommand::redo (void)
 		}
 	}
 
+	// Let same old statistics prevail...
+	pNewTrack->setMidiNoteMax(pTrack->midiNoteMax());
+	pNewTrack->setMidiNoteMin(pTrack->midiNoteMin());
+
 	// About to copy all automation/curves...
 	qtractorCurveList *pCurveList = pTrack->curveList();
 	qtractorCurveList *pNewCurveList = pNewTrack->curveList();
@@ -532,8 +545,6 @@ bool qtractorMoveTrackCommand::redo (void)
 	// Just insert under the track list position...
 	// We'll renumber all items now...
 	iNextTrack = pTrackList->insertTrack(iNextTrack, pTrack);
-	if (iNextTrack >= 0)
-		pTrackList->setCurrentTrackRow(iNextTrack);
 
 	// Swap it nice, finally.
 	m_pNextTrack = pNextTrack;
@@ -543,6 +554,9 @@ bool qtractorMoveTrackCommand::redo (void)
 	if (pMixer)
 		pMixer->updateTracks(true);
 
+	// Make it new current track (updates mixer too)...
+	pTrackList->setCurrentTrackRow(iNextTrack);
+
 	// ATTN: MIDI controller map feedback.
 	qtractorMidiControl *pMidiControl = qtractorMidiControl::getInstance();
 	if (pMidiControl) {
@@ -785,6 +799,11 @@ bool qtractorEditTrackCommand::redo (void)
 			pClip->open();
 	}
 
+	// Mixer turn...
+	qtractorMixer *pMixer = pMainForm->mixer();
+	if (pMixer)
+		pMixer->updateTracks(true);
+
 	// Finally update any outstanding clip editors...
 	m_pTrack->updateClipEditors();
 
@@ -1069,11 +1088,18 @@ bool qtractorTrackMonitorCommand::redo (void)
 	// Set track monitoring...
 	pTrack->setMonitor(m_bMonitor);
 
+	// Send MMC MASKED_WRITE command...
+	qtractorMidiEngine *pMidiEngine = pSession->midiEngine();
+	const int iTrack = pSession->tracks().find(pTrack);
+	if (pMidiEngine) {
+		pMidiEngine->sendMmcMaskedWrite(
+			qtractorMmcEvent::TRACK_MONITOR, iTrack, m_bMonitor);
+	}
+
 	// Send MIDI controller command(s)...
 	if (midiControlFeedback()) {
 		qtractorMidiControl *pMidiControl = qtractorMidiControl::getInstance();
 		if (pMidiControl) {
-			const int iTrack = pSession->tracks().find(pTrack);
 			pMidiControl->processTrackCommand(
 				qtractorMidiControl::TRACK_MONITOR, iTrack, m_bMonitor);
 		}
@@ -1336,6 +1362,7 @@ bool qtractorTrackInstrumentCommand::redo (void)
 		iBankSelMethod, m_iBank, m_iProg, pTrack);
 
 	// Set track instrument...
+	pTrack->setMidiBankSelMethod(iBankSelMethod);
 	pTrack->setMidiBank(m_iBank);
 	pTrack->setMidiProg(m_iProg);
 
