@@ -1,7 +1,7 @@
 // qtractorTrackForm.cpp
 //
 /****************************************************************************
-   Copyright (C) 2005-2017, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2005-2018, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -239,6 +239,9 @@ qtractorTrackForm::qtractorTrackForm (
 	QObject::connect(m_ui.BankSelMethodComboBox,
 		SIGNAL(activated(int)),
 		SLOT(bankSelMethodChanged(int)));
+	QObject::connect(m_ui.DrumsCheckBox,
+		SIGNAL(clicked()),
+		SLOT(changed()));
 	QObject::connect(m_ui.BankComboBox,
 		SIGNAL(editTextChanged(const QString&)),
 		SLOT(bankChanged()));
@@ -359,6 +362,7 @@ void qtractorTrackForm::setTrack ( qtractorTrack *pTrack )
 	m_ui.ChannelSpinBox->setValue(m_props.midiChannel + 1);
 	updateChannel(m_ui.ChannelSpinBox->value(),
 		m_props.midiBankSelMethod, m_props.midiBank, m_props.midiProg);
+	m_ui.DrumsCheckBox->setChecked(m_props.midiDrums);
 
 	// Update colors...
 	updateColorItem(m_ui.ForegroundColorComboBox, m_props.foreground);
@@ -439,6 +443,7 @@ void qtractorTrackForm::accept (void)
 		m_props.midiBankSelMethod = m_ui.BankSelMethodComboBox->currentIndex();
 		m_props.midiBank = midiBank();
 		m_props.midiProg = midiProg();
+		m_props.midiDrums = m_ui.DrumsCheckBox->isChecked();
 		// View colors...
 		m_props.foreground = colorItem(m_ui.ForegroundColorComboBox);
 		m_props.background = colorItem(m_ui.BackgroundColorComboBox);
@@ -1153,26 +1158,28 @@ void qtractorTrackForm::trackIconClicked (void)
 	if (sFilename.at(0) == ':')
 		sFilename.clear();
 
+	const QString& sTitle
+		= tr("Track Icon") + " - " QTRACTOR_TITLE;
+
 	QStringList filters;
 	filters.append(tr("Image files (%1)").arg("*.png *.xpm *.jpg *.jpeg"));
 	filters.append(tr("All files (*.*)"));
+	const QString& sFilter = filters.join(";;");
 
-	const QString& sTitle
-		= tr("Track Icon") + " - " QTRACTOR_TITLE;
-	const QString& sFilter
-		= filters.join(";;");
-
+	QWidget *pParentWidget = NULL;
 	QFileDialog::Options options = 0;
 	qtractorOptions *pOptions = qtractorOptions::getInstance();
-	if (pOptions && pOptions->bDontUseNativeDialogs)
+	if (pOptions && pOptions->bDontUseNativeDialogs) {
 		options |= QFileDialog::DontUseNativeDialog;
+		pParentWidget = QWidget::window();
+	}
 #if 1//QT_VERSION < 0x040400
 	// Ask for the filename to open...
-	sFilename = QFileDialog::getOpenFileName(this,
+	sFilename = QFileDialog::getOpenFileName(pParentWidget,
 		sTitle, sFilename, sFilter, NULL, options);
 #else
 	// Construct open-file dialog...
-	QFileDialog fileDialog(this, sTitle, sFilename, sFilter);
+	QFileDialog fileDialog(pParentWidget, sTitle, sFilename, sFilter);
 	// Set proper open-file modes...
 	fileDialog.setAcceptMode(QFileDialog::AcceptOpen);
 	fileDialog.setFileMode(QFileDialog::AnyFile);
@@ -1391,7 +1398,8 @@ void qtractorTrackForm::progChanged (void)
 	// Of course, only applicable on MIDI tracks...
 	if (m_pMidiBus) {
 		// Patch parameters...
-		const unsigned short iChannel = m_ui.ChannelSpinBox->value() - 1;
+		const unsigned short iChannel
+			= m_ui.ChannelSpinBox->value() - 1;
 		QString sInstrumentName;
 		if (m_ui.InstrumentComboBox->currentIndex() > 0)
 			sInstrumentName = m_ui.InstrumentComboBox->currentText();
@@ -1420,6 +1428,17 @@ void qtractorTrackForm::progChanged (void)
 		// Patch it directly...
 		m_pMidiBus->setPatch(iChannel, sInstrumentName,
 			iBankSelMethod, iBank, iProg, m_pTrack);
+		// Whether in drum-mode auto-magically...
+		qtractorSession *pSession = qtractorSession::getInstance();
+		if (pSession) {
+			qtractorInstrumentList *pInstruments = pSession->instruments();
+			if (pInstruments) {
+				const qtractorInstrument& instr
+					= pInstruments->value(sInstrumentName);
+				m_ui.DrumsCheckBox->setChecked(
+					iChannel == 9 || instr.isDrum(iBank, iProg));
+			}
+		}
 		// Make it dirty.
 		++m_iDirtyPatch;
 	}
@@ -1435,13 +1454,17 @@ void qtractorTrackForm::selectForegroundColor (void)
 	const QString& sTitle
 		= tr("Foreground Color") + " - " QTRACTOR_TITLE;
 
+	QWidget *pParentWidget = NULL;
 	QColorDialog::ColorDialogOptions options = 0;
 	qtractorOptions *pOptions = qtractorOptions::getInstance();
-	if (pOptions && pOptions->bDontUseNativeDialogs)
+	if (pOptions && pOptions->bDontUseNativeDialogs) {
 		options |= QColorDialog::DontUseNativeDialog;
+		pParentWidget = QWidget::window();
+	}
 
 	const QColor& color = QColorDialog::getColor(
-		colorItem(m_ui.ForegroundColorComboBox), this, sTitle, options);
+		colorItem(m_ui.ForegroundColorComboBox),
+		pParentWidget, sTitle, options);
 
 	if (color.isValid()) {
 		m_props.foreground = color;
@@ -1457,13 +1480,17 @@ void qtractorTrackForm::selectBackgroundColor (void)
 	const QString& sTitle
 		= tr("Background Color") + " - " QTRACTOR_TITLE;
 
+	QWidget *pParentWidget = NULL;
 	QColorDialog::ColorDialogOptions options = 0;
 	qtractorOptions *pOptions = qtractorOptions::getInstance();
-	if (pOptions && pOptions->bDontUseNativeDialogs)
+	if (pOptions && pOptions->bDontUseNativeDialogs) {
 		options |= QColorDialog::DontUseNativeDialog;
+		pParentWidget = QWidget::window();
+	}
 
 	const QColor& color = QColorDialog::getColor(
-		colorItem(m_ui.BackgroundColorComboBox), this, sTitle, options);
+		colorItem(m_ui.BackgroundColorComboBox),
+		pParentWidget, sTitle, options);
 
 	if (color.isValid()) {
 		m_props.background = color;
